/*
 *     This file is part of the Redmine requirement connector for Squash TM (henix-tm-redmine-requirements) project.
 *     Copyright (C) 2017 - 2022 Henix, henix.fr - All Rights Reserved
 *
 *     Unauthorized copying of this file, via any medium is strictly prohibited
 *     Proprietary and confidential
 *
 * 	 (C)Henix. Tous droits réservés.
 *
 * 	Avertissement : ce programme est protégé par la loi relative au droit d'auteur et par les conventions internationales. Toute reproduction ou distribution partielle ou totale du logiciel, par quelque moyen que ce soit, est strictement interdite.
 */
require(['common'], function () {

	require(["jquery", "app/pubsub", "app/ws/squashtm.notification", "underscore", "squash.configmanager",
			"squash.basicwidgets", "squash.translator", "app/util/PathUtils", "squash.attributeparser", "serverauth/auth-manager",
			"app/ws/squashtm.workspace", "jquery.squash.togglepanel", "squashtable", "jquery.squash.formdialog", "jquery.switchButton"],
		function ($, pubsub, notification, _, confman, basic, translator, pathUtils, attrparser, authmanager) {

			// *********** Prefiling the ajax with the required token (Squash 1.18+) ***********

			if ($("meta[name='_csrf']").length > 0) {
				$.ajaxPrefilter(function (options, originalOptions, jqXHR) {
					var token = $("meta[name='_csrf']").attr("content");
					var header = $("meta[name='_csrf_header']").attr("content");
					jqXHR.setRequestHeader(header, token);
				});
			}

			// *************** EVENTS SUBSCRIPTION ***************

			pubsub.subscribe("redminereq.configuration.information", configureInfoPanel);
			pubsub.subscribe("redminereq.configuration.filtertable", configureProjectsPanel);
			pubsub.subscribe("redminereq.configuration.options", configureOptionsPanel);
			pubsub.subscribe("redminereq.configuration.mappingtable", configureMappingsPanel);
			pubsub.subscribe("redminereq.configuration.linktable", configureLinksPanel);
			pubsub.subscribe("redminereq.configuration.valuemappings", configureValuesMappingPanel);
			pubsub.subscribe("redminereq.configuration.dialogs", configurePopups);
			pubsub.subscribe("redminereq.configuration.complete", complete);


			// *************** INFORMATION PANEL CONFIGURATION ***************

			function configureInfoPanel() {
				configureServerSelector();
			}

			// ** server selector configuration **
			function configureServerSelector() {
				var globconf = squashtm.plugin.redminereq.conf;
				var server = $("#redminereq-server");

				var cfg = confman.getJeditableSelect();
				cfg.data = confman.toJeditableSelectFormat(globconf.servers);
				cfg.data.selected = globconf.configuration.serverId;
				cfg.style = "display:inline";

				var label = cfg.data[globconf.configuration.serverId];
				server.text(label);

				var url = baseUrl() + '/server';

				server.editable(url, cfg);
			}


			// *************** PROJECTS PANEL CONFIGURATION ***************

			function configureProjectsPanel() {
				configureFilterTable();
			}

			// ** filter table configuration **
			function configureFilterTable() {
				var table = $("#redminereq-filters-table");
				var configuration = squashtm.plugin.redminereq.conf.configuration;

				// validation function
				function onsubmit(settings, editable) {

					var error = "no-error";

					var $this = $(this);
					var td = $this.parent().get(0);
					var row = td.parentNode;

					var field = getfield(td);
					var val = $this.find('input').val();

					// build a new filterbinding instance and validate it
					var binding = $.extend({}, table.fnGetData(row));

					/*
					 * cases where a modification is valid :
					 * 1/ path is whatever you want and
					 * 2/ the field wasn't modified
					 */
					var changedpath = (field === "path"),
						nochange = (binding[field] === val);

					// if the situation is none of the above, call validation
					if (!(changedpath || nochange)) {
						binding[field] = val;
						binding.id = binding.key + "+" + binding.filter;
						error = table.validate(binding);
					} else if (changedpath) {
						if (!pathUtils.validatePartialPath(val)) {
							error = "path-error";
						}
					}

					// show a warning if no good
					var valid = false,
						errmsg = "";

					switch (error) {
						case "project-empty" :
							errmsg = 'henix.redminereq.filters.dialog.add.projectempty';
							break;
						case "filter-empty" :
							errmsg = 'henix.redminereq.filters.dialog.add.filterempty';
							break;
						// 6979 - error message for project empty AND filter empty
						case "project-empty filter-empty" :
							errmsg = 'henix.redminereq.filters.dialog.add.projectempty henix.redminereq.filters.dialog.add.filterempty';
							break;
						case "filter-noInt" :
							errmsg = 'henix.redminereq.filters.dialog.add.filternoint';
							break;
						case "pair-exists" :
							errmsg = 'henix.redminereq.filters.dialog.add.nonunique';
							break;
						case "path-error" :
							errmsg = 'henix.redminereq.filters.dialog.add.pathinvalid';
							break;
						default :
							valid = true;
							break;
					}

					if (!valid) {
						if (errmsg.indexOf(' ') > -1) {
							var errorMessage = errmsg.split(' ', 2);
							notification.showError(translator.get(errorMessage[0]));
							notification.showError(translator.get(errorMessage[1]));
						} else {
							notification.showError(translator.get(errmsg));
						}
					}

					return valid;
				}


				function oncomplete(value, settings) {
					var td = this;
					var row = td.parentNode;
					var field = getfield(td);
					var data = table.fnGetData(row);

					data[field] = value;
					data.id = data.key + "+" + data.filter;

					table.fnUpdate(data, row);
				}

				var postfunction = makePostfunction(baseUrl() + '/filters', table);

				table.squashTable({
						bServerSide: false,
						aaData: configuration.filterBindings,

						fnDrawCallback: function () {
							var edconf = confman.getStdJeditable(),
								cells = table.find('tbody').find('.redminereq-key, .redminereq-filter, .redminereq-path');


							// validation callback
							edconf.onsubmit = onsubmit;

							// table update callback
							edconf.callback = oncomplete;

							cells.editable(postfunction, edconf);
						}
					},

					{
						functions: {
							// because the native function doesn't allow non-numeric ids, we have to override it
							getODataId: function (arg) {
								return this.fnGetData(arg)['id'];
							},

							validate: function (binding) {
								var projkey = binding.key;
								if ($.trim(projkey) === "" && $.trim(projfilter) != "") {
									return "project-empty";
								}

								var projfilter = binding.filter;
								if ($.trim(projfilter) === "" && $.trim(projkey) != "") {
									return "filter-empty";
								}

								if ($.trim(projfilter) === "" && $.trim(projfilter) === "") {
									return "project-empty filter-empty";
								}

								if (Math.floor(projfilter) != projfilter) {
									return "filter-noInt";
								}

								// if such binding already exists : fail
								// (we really mean : project key / filter duplicates are not allowed)
								var exists = this.getDataById(binding.id);
								if (exists !== null) {
									return "pair-exists";
								}

								var path = binding.path;
								if (!pathUtils.validatePartialPath(path)) {
									return "path-error";
								}

								// else, no error
								return "no-error";
							},

							deleteRowsByIds: function (ids) {
								var self = this;
								var rows = self.getRowsByIds(ids);
								rows.each(function () {
									self.fnDeleteRow(this, null, false);
								});
								self.fnDraw();
							}
						}
					});
			}


			// *************** OPTIONS PANEL CONFIGURATION ***************

			function configureOptionsPanel() {
				configureAllowToDeleteReqOption();
			}

			// ** configuration of the option to delete or change status to 'obsolete' for synchronized requirements **
			function configureAllowToDeleteReqOption() {
				var activCbx = $("#toggle-activation-checkbox");
				var activConf = attrparser.parse(activCbx.data('def'));
				activCbx.switchButton(activConf);
				// a bit of css tweak now for aligning label and switchButton
				$("#switch-button").css({position: 'relative', top: '8px'});
				activCbx.siblings('.switch-button-background').css({position: 'relative', top: '5px'});
				// explanation box
				$("#delete-req-explanation").css("font-size", "10px").css("padding-left", "5px").css("margin", "8px 0px 5px 25px").css("border-left", "black solid 2px");

				var allowToDeleteReq = squashtm.plugin.redminereq.conf.configuration.allowToDeleteReq;
				(allowToDeleteReq) ? activCbx.switchButton({checked: true}) : activCbx.switchButton({checked: false});


				activCbx.on('change', function () {
					var shouldActivate = $("#toggle-activation-checkbox").prop('checked');
					if (shouldActivate) {
						$.ajax({
							url: baseUrl() + '/options',
							type: 'POST',
							contentType: 'application/json',
							data: JSON.stringify(true)
						}).success(function () {
							activCbx.switchButton({checked: true});
						});
					}
					else {
						$.ajax({
							url: baseUrl() + '/options',
							type: 'POST',
							contentType: 'application/json',
							data: JSON.stringify(false)
						}).success(function () {
							activCbx.switchButton({checked: false});
						});
					}
				});
			}


			// *************** MAPPINGS PANEL CONFIGURATION ***************

			function configureMappingsPanel() {
				configureMappingTable();
			}

			function configureMappingTable() {
				var globconf = squashtm.plugin.redminereq.conf,
					configuration = globconf.configuration,
					table = $("#redminereq-mappings-table"),
					allFields = globconf.availableFields;

				var postfunction = makePostfunction(baseUrl() + '/mappings', table);

				// this function creates the list of available items in the dropdown menu
				// note that in the context of an (optional) given row, the ID of that row will still be included in the dropdown
				var mkFieldlist = function (ctxt) {
					var allids = table.squashTable().fnGetData().map(function (data) {
						return data.id
					});

					var effective = _.omit(allFields, allids);

					// Special case : if a context is supplied (a table row), add back the key-value it represents
					// and make it the default for it
					if (ctxt !== undefined) {
						rId = table.fnGetData(ctxt).id;
						effective[rId] = allFields[rId];
						effective.selected = rId;
					}

					return effective;
				};

				function squashFieldCallback(value, settings) {
					var td = this;
					var row = td.parentNode;
					var data = table.fnGetData(row);

					data.squashField = value;
					data.id = data.squashField;

					table.fnUpdate(data, row);
					table.fnDraw();	// this will reinitialize the content of jeditables
				}

				table.squashTable({
					bServerSide: false,
					aoColumnDefs: [{aTargets: [0], sWidth: '1.5%'}, {aTargets: [1], sWidth: '38.5%'}, {
						aTargets: [2],
						sWidth: '55%'
					}, {aTargets: [3], sWidth: '5%'}],
					aaData: configuration.fieldMappings,
					fnDrawCallback: function () {

						var self = this;

						var rows = table.fnGetNodes();
						rows.forEach(function (row) {
							var $row = $(row),
								sqcell = $row.find('td.redminereq-squashfield'),
								rcell = $row.find('td.redminereq-redminefield');

							sqcell.text(allFields[sqcell.text()]);

							// now the features for the editable rows
							var locked = table.fnGetData(row).locked;
							if (!locked) {
								// configure the dropdown for the squash field
								var sconf = confman.getJeditableSelect();
								sconf.data = mkFieldlist(row);
								sconf.callback = squashFieldCallback;
								sqcell.editable(postfunction, sconf);

								// configure the text editable for redmine field
								var rconf = confman.getStdJeditable();
								rcell.editable(postfunction, rconf);
							}
						});
					}
				}, {
					// because not all rows can be deleted
					// we must configure a custom handling of deletion
					buttons: [{
						tdSelector: 'td.delete-mapping-button',
						uiIcon: function (row, data) {
							return (data.locked) ? 'ui-icon-locked' : 'ui-icon-minus';
						},
						disabled: function (row, data) {
							return data.locked;
						},
						jquery: true,
						onClick: function (table, cell) {
							var row = $(cell).parents('tr').get(0);
							var id = table.getODataId(row);
							var popup = $("#redminereq-mappings-delete-dialog");
							popup.data('entity-id', id);
							popup.formDialog('open');
							$("#redminereq-mappings-delete-dialog-label").text(translator.get('henix.redminereq.configuration.mappings.dialog.delete.message.row'));
						}
					}],

					functions: {
						// because the native function doesn't allow non-numeric ids, we have to override it
						getODataId: function (arg) {
							return this.fnGetData(arg)['id'];
						},

						deleteRowsByIds: function (ids) {
							var self = this;
							var rows = self.getRowsByIds(ids);
							rows.each(function () {
								if (!self.fnGetData(this).locked) {
									self.fnDeleteRow(this, null, false);
								}
							});
							self.fnDraw();
						}
					}
				});

				// bind the '+' button
				$("#redminereq-mappings-add-button").on('click', function () {
					var available = mkFieldlist();
					if (_.isEmpty(available)) {
						notification.showWarning(translator.get('henix.redminereq.configuration.mappings.allaremapped'));
					}
					else {
						var firstAvailable = _.keys(available)[0];
						var mapping = {
							id: firstAvailable,
							squashField: firstAvailable,
							redmineField: "",
							locked: false
						};

						$.ajax({
							url: baseUrl() + '/mappings',
							type: 'POST',
							contentType: 'application/json',
							data: JSON.stringify(mapping)
						})
							.success(function () {
								table.squashTable().fnAddData(mapping);
							});


					}
				});

			}


			// *************** LINKS PANEL CONFIGURATION ***************

			function configureLinksPanel() {
				configureLinkTable();
			}

			function configureLinkTable() {
				var globconf = squashtm.plugin.redminereq.conf,
					configuration = globconf.configuration,
					table = $("#redminereq-links-table"),
					allFields = globconf.availableRedmineLinksFields;

				var postfunction = makePostfunctionForLinks(baseUrl() + '/links', table);

				var mkFieldlist = function (ctxt) {
					var allids = table.squashTable().fnGetData().map(function (data) {
						return data.id
					});

					var effective = _.omit(allFields, allids);

					if (ctxt !== undefined) {
						rId = table.fnGetData(ctxt).id;
						effective[rId] = allFields[rId];
						effective.selected = rId;
					}
					return effective;
				};

				function squashFieldCallback(value, settings) {
					var td = this;
					var row = td.parentNode;
					var data = table.fnGetData(row);

					data.squashField = globconf.linkTypes[value];

					// 6982 - return to normal
					$(row).find('td.redminereq-squashfield').css('color', 'black').css('font-style', 'normal').css('font-size', '1em');

					table.fnUpdate(data, row);
					table.fnDraw();
				}

				table.squashTable({
					bServerSide: false,
					aoColumnDefs: [{aTargets: [0], sWidth: '1.5%'}, {aTargets: [1], sWidth: '38.5%'}, {
						aTargets: [2],
						sWidth: '55%'
					},
						{
							aTargets: [3], sWidth: '5%', mRender: function (type, data, full) {
								return '<a></a>';
							}
						}],
					aaData: configuration.fieldLinks,
					fnDrawCallback: function () {

						var self = this;

						var rows = table.fnGetNodes();
						rows.forEach(function (row) {
							var $row = $(row),
								sqcell = $row.find('td.redminereq-squashfield'),
								rcell = $row.find('td.redminereq-redminefield');

							var confcell = $row.find('.reset-link-button');
							confcell.text(translator.get('henix.redminereq.configuration.links.table.label.erase'));
							confcell.css("text-decoration", "underline").css("cursor", "pointer");
							confcell.on('click', function () {
								var row = $(this).parents('tr').get(0);
								var id = $("#redminereq-links-table").squashTable().getODataId(row);
								var popup = $("#redminereq-links-reset-dialog");
								popup.data('entity-id', id);
								popup.formDialog('open');
								$("#redminereq-links-reset-dialog-label").text(translator.get('henix.redminereq.configuration.links.dialog.reset.message.row'));
							});

							rcell.text(allFields[rcell.text()]);
							sqcell.text(globconf.linkTypes[sqcell.text()]);

							// 6982 - css trick for "(Click to edit...)", standardise the label
							if (sqcell.text() === "") {
								sqcell.css('font-size', '0.9em').css('font-style', 'italic').css('color', '#999999');
							}

							// configure the dropdown for the squash field
							var sconf = confman.getJeditableSelect();
							sconf.data = globconf.linkTypes;
							sconf.callback = squashFieldCallback;
							sqcell.editable(postfunction, sconf);
						});
					}
				}, {
					functions: {
						// because the native function doesn't allow non-numeric ids, we have to override it
						getODataId: function (arg) {
							return this.fnGetData(arg)['id'];
						},
						resetRowsByIds: function (ids) {
							var self = this;
							var rows = self.getRowsByIds(ids);
							rows.each(function () {
								var data = self.fnGetData($(this));

								data.squashField = null;

								self.fnUpdate(data, $(this));
								self.fnDraw();	// this will reinitialize the content of jeditables

							});

						}
					}

				});
			}


			function makePostfunctionForLinks(baseurl, table) {
				return function (value, settings) {
					var td = this;
					var globconf = squashtm.plugin.redminereq.conf;
					var row = td.parentNode;
					var field = getfield(td);
					var rowmodel = table.fnGetData(row);

					var url = baseurl + '/' + rowmodel.id + '/' + field;

					$.ajax({
						url: url,
						type: 'POST',
						data: {value: globconf.linkTypes[value]}
					});

					return value;
				}
			}

			function makePostfunctionForFilters(baseurl, table) {
				return function (value, settings) {
					var td = this;
					var globconf = squashtm.plugin.redminereq.conf;
					var row = td.parentNode;
					var field = getfield(td);
					var rowmodel = table.fnGetData(row);

					var url = baseurl + '/' + rowmodel.id + '/' + field;

					$.ajax({
						url: url,
						type: 'POST',
						data: {value: globconf.linkTypes[value]}
					});

					return value;
				}
			}

			// *************** CUSTOM VALUE MAPPING PANEL CONFIGURATION ***************

			function configureValuesMappingPanel() {
				configureValuesMapping();
			}

			function configureValuesMapping() {
				var saveBtn = $("#redminereq-value-mappings-save"),
					restoreBtn = $("#redminereq-value-mappings-restore"),
					textarea = $("#redminereq-value-mappings-text");


				/*
				 * Logic binding the textarea, the save and restore button :
				 * - restore btn starts its lifecycle disabled.
				 * - on textarea content chage - > restore button is enabled
				 * - on successful save or successful restore -> restore button is disabled
				 */


				// enable TAB in the text area,
				// trigger save on ENTER,
				// activate the restore button on first change
				// same for save reminder message
				// credits goes to http://jsfiddle.net/2wAzx/13/
				textarea.on('keydown', function (evt) {

					//activate the restore button
					restoreBtn.removeClass('not-displayed');
					saveBtn.removeClass("not-displayed");

					var key = evt.keyCode || evt.which;

					// 9 is TAB
					if (key === 9) {
						// get caret position/selection
						var val = this.value,
							start = this.selectionStart,
							end = this.selectionEnd;

						// set textarea value to: text before caret + spaces + text after caret
						// note : 'tab' ('\t') is not a valid yaml character so we will insert tree spaces instead
						this.value = val.substring(0, start) + '   ' + val.substring(end);

						// put caret at right position again
						this.selectionStart = this.selectionEnd = start + 3;

						// prevent the focus lose
						return false;
					}

				});

				// save action
				saveBtn.on('click', function () {
					var txt = textarea.val();
					$.ajax({
						url: baseUrl() + '/valuemappings',
						type: 'POST',
						data: {value: txt}
					})
						.success(function () {
							saveBtn.addClass('not-displayed');
							restoreBtn.addClass('not-displayed');
						});
				});

				//restore action
				restoreBtn.on('click', function () {
					$.get(baseUrl() + '/valuemappings')
						.success(function (yaml) {
							textarea.val(yaml);
							saveBtn.addClass('not-displayed');
							restoreBtn.addClass('not-displayed');
						});
				});


			}


			// *************** POPUPS CONFIGURATION ***************

			function configurePopups() {
				configureAddfilterDialog();
				configureRemoveFilterDialog();
				configureRemoveSelectedFilterDialog();
				configureRemoveMappingDialog();
				configureRemoveSelectedMappingDialog();
				configureResetLinkDialog();
				configureResetSelectedLinkDialog();
			}

			function configureAddfilterDialog() {

				var table = $("#redminereq-filters-table");

				var dialog = $("#redminereq-filters-add-dialog");
				dialog.formDialog();

				function hideWarnings() {
					dialog.find('.redminereq-invalid-project-empty, .redminereq-invalid-filter-empty, .redminereq-invalid-filter-noInt, .redminereq-invalid-pair-exists, .redminereq-invalid-path-error').hide();
				}

				// open, cleanup
				dialog.on('formdialogopen', function () {
					hideWarnings();
				});

				// confirmation
				dialog.on('formdialogconfirm', function () {

					hideWarnings();

					var key = $.trim($("#redminereq-filter-add-key").val()),
						filter = $.trim($("#redminereq-filter-add-filter").val()),
						path = $.trim($("#redminereq-filter-add-path").val());

					var binding = {
						id: key + "+" + filter,
						key: key,
						filter: filter,
						path: path
					};

					var error = table.squashTable().validate(binding);

					if (error === "no-error") {
						$.ajax({
							url: baseUrl() + '/filters',
							type: 'POST',
							data: JSON.stringify(binding),
							contentType: 'application/json'
						})
							.success(function () {
								dialog.formDialog('close');
								table.squashTable().fnAddData(binding);
							});
					}
					else {
						// 6979 - error message for project empty AND filter empty
						if (error.indexOf(' ') > -1) {
							var errorMultiple = error.split(' ', 2);
							dialog.find('.redminereq-invalid-' + errorMultiple[0]).show();
							dialog.find('.redminereq-invalid-' + errorMultiple[1]).show();
						} else {
							dialog.find('.redminereq-invalid-' + error).show();
						}
					}
				});

				// rejection
				dialog.on('formdialogcancel', function () {
					dialog.formDialog('close');
				});

				// opening
				$("#redminereq-filters-add-button").on('click', function () {
					var globconf = squashtm.plugin.redminereq.conf;
					var serverId = _.find(globconf.servers, function (server) {
						return server.name == $("#redminereq-server").text()
					}).id;
					if (serverId !== 0) {
						dialog.formDialog('open');
					} else {
						notification.showWarning(translator.get('henix.redminereq.configuration.filters.no.server'));
					}
				});

				return dialog;
			}


			function configureRemoveFilterDialog() {

				var table = $("#redminereq-filters-table");
				var dialog = $("#redminereq-filters-delete-dialog");

				dialog.formDialog();

				dialog.on('formdialogcancel', function () {
					dialog.formDialog('close');
				});

				var deleteconfirm = makeConfirmDelete(table, dialog, baseUrl() + '/filters');
				dialog.on('formdialogconfirm', deleteconfirm);


				return dialog;
			}

			function configureRemoveSelectedFilterDialog() {

				var table = $("#redminereq-filters-table");
				var dialog = $("#redminereq-filters-delete-selected-dialog");

				dialog.formDialog();

				dialog.on('formdialogcancel', function () {
					dialog.formDialog('close');
				});

				var deleteconfirm = makeConfirmDeleteSelected(table, dialog, baseUrl() + '/filters');
				dialog.on('formdialogconfirm', deleteconfirm);

				$("#redminereq-filters-delete-button").on('click', function () {
					var select = table.squashTable().getSelectedIds();
					if (select.length === 0) {
						notification.showWarning(translator.get('message.EmptyTableSelection'));
					} else if (select.length === 1) {
						dialog.formDialog('open');
						$("#redminereq-filters-delete-selected-dialog-label").text(translator.get('henix.redminereq.configuration.filters.dialog.delete.message.row'));
					} else {
						dialog.formDialog('open');
						$("#redminereq-filters-delete-selected-dialog-label").text(translator.get('henix.redminereq.configuration.filters.dialog.delete.message.rows'));
					}
				});


				return dialog;
			}


			function configureResetLinkDialog() {
				var table = $("#redminereq-links-table");
				var dialog = $("#redminereq-links-reset-dialog");

				dialog.formDialog();
				dialog.on('formdialogcancel', function () {
					dialog.formDialog('close');
				});

				var resetconfirm = makeConfirmReset(table, dialog, baseUrl() + '/links');
				dialog.on('formdialogconfirm', resetconfirm);


			}

			function configureResetSelectedLinkDialog() {
				var table = $("#redminereq-links-table");
				var dialog = $("#redminereq-links-reset-selected-dialog");

				dialog.formDialog();
				dialog.on('formdialogcancel', function () {
					dialog.formDialog('close');
				});

				var resetconfirm = makeConfirmResetSelected(table, dialog, baseUrl() + '/links');
				dialog.on('formdialogconfirm', resetconfirm);

				$("#redminereq-links-reset-btn").on('click', function () {
					var select = table.squashTable().getSelectedIds();

					if (select.length === 0) {
						notification.showWarning(translator.get('henix.redminereq.configuration.links.dialog.reset.empty'));
					} else if (select.length === 1) {
						dialog.formDialog('open');
						$("#redminereq-links-reset-selected-dialog-label").text(translator.get('henix.redminereq.configuration.links.dialog.reset.message.row'));
					} else {
						dialog.formDialog('open');
						$("#redminereq-links-reset-selected-dialog-label").text(translator.get('henix.redminereq.configuration.links.dialog.reset.message.rows'));
					}
				});
			}


			function configureRemoveMappingDialog() {

				var table = $("#redminereq-mappings-table");
				var dialog = $("#redminereq-mappings-delete-dialog");

				dialog.formDialog();

				dialog.on('formdialogcancel', function () {
					dialog.formDialog('close');
				});

				var deleteconfirm = makeConfirmDelete(table, dialog, baseUrl() + '/mappings');
				dialog.on('formdialogconfirm', deleteconfirm);


				return dialog;
			}

			function configureRemoveSelectedMappingDialog() {

				var table = $("#redminereq-mappings-table");
				var dialog = $("#redminereq-mappings-delete-selected-dialog");

				dialog.formDialog();

				dialog.on('formdialogcancel', function () {
					dialog.formDialog('close');
				});

				var deleteconfirm = makeConfirmDeleteSelected(table, dialog, baseUrl() + '/mappings');
				dialog.on('formdialogconfirm', deleteconfirm);

				$("#redminereq-mappings-delete-button").on('click', function () {
					var select = table.squashTable().getSelectedIds();
					var lockedIds = ["CATEGORY", "STATUS", "DESCRIPTION", "CRITICALITY", "REFERENCE", "LABEL"];
					var isLockedIdsSelected = lockedIds.filter(function (elem) {
						return select.indexOf(elem) > -1;
					}).length;
					if (select.length === 0) {
						notification.showWarning(translator.get('message.EmptyTableSelection'));
					} else if (select.length === 1) {
						if (isLockedIdsSelected > 0) {
							notification.showWarning(translator.get('henix.redminereq.configuration.mappings.dialog.delete.message.locked.selected.row'));
						} else {
							dialog.formDialog('open');
							$("#redminereq-mappings-delete-selected-dialog-label").text(translator.get('henix.redminereq.configuration.mappings.dialog.delete.message.row'));
						}
					} else {
						if (isLockedIdsSelected > 0 && isLockedIdsSelected < select.length) {
							dialog.formDialog('open');
							$("#redminereq-mappings-delete-selected-dialog-label").text(translator.get('henix.redminereq.configuration.mappings.dialog.delete.message.locked-unlocked.rows'));
						} else if (isLockedIdsSelected > 0 && isLockedIdsSelected === select.length) {
							notification.showWarning(translator.get('henix.redminereq.configuration.mappings.dialog.delete.message.locked.selected.rows'));
						} else {
							dialog.formDialog('open');
							$("#redminereq-mappings-delete-selected-dialog-label").text(translator.get('henix.redminereq.configuration.mappings.dialog.delete.message'));
						}
					}

				});

				return dialog;
			}


			// *************** FINAL INITIALIZATION ***************

			function complete() {
				basic.init();
			}


			// *************** COMMON CODE ***************

			function baseUrl() {
				var globconf = squashtm.plugin.redminereq.conf;
				return squashtm.app.contextRoot + 'redminereq/project/' + globconf.projectId + '/configuration';
			}

			// looks for 'key' in css class redminereq-key, 'filter' in 'redminereq-filter' etc
			function getfield(td) {
				var classes = td.className;
				return /redminereq-(\w+)/.exec(classes)[1];
			}

			function makePostfunction(baseurl, table) {
				return function (value, settings) {
					var td = this;
					var row = td.parentNode;
					var field = getfield(td);
					var rowmodel = table.fnGetData(row);

					var url = baseurl + '/' + field;

					$.ajax({
						url: url,
						type: 'POST',
						data: {
							value: value,
							id: rowmodel.id
						}
					});

					return value;
				}
			}

			function makeConfirmDelete(table, dialog, baseurl) {
				return function () {
					var ids;
					var id = dialog.data('entity-id');
					if (!!id) {
						ids = [id];
					}
					else {
						ids = table.squashTable().getSelectedIds();
					}

					$.ajax({
						url: baseurl + '/' + ids.join(','),
						type: 'DELETE'
					})
						.success(function () {
							dialog.formDialog('close');
							table.squashTable().deleteRowsByIds(ids);
						});
				}
			}

			function makeConfirmDeleteSelected(table, dialog, baseurl) {
				return function () {
					var ids;
					ids = table.squashTable().getSelectedIds();

					$.ajax({
						url: baseurl + '/' + ids.join(','),
						type: 'DELETE'
					})
						.success(function () {
							dialog.formDialog('close');
							table.squashTable().deleteRowsByIds(ids);
						});
				}
			}

			function makeConfirmReset(table, dialog, baseurl) {
				return function () {
					var ids;
					var id = dialog.data('entity-id');
					ids = [id];

					$.ajax({
						url: baseurl + '/' + ids.join(','),
						type: 'POST'
					})
						.success(function () {
							dialog.formDialog('close');
							table.squashTable().resetRowsByIds(ids);
						});
				}
			}

			function makeConfirmResetSelected(table, dialog, baseurl) {
				return function () {
					var ids;
					ids = table.squashTable().getSelectedIds();

					$.ajax({
						url: baseurl + '/' + ids.join(','),
						type: 'POST'
					})
						.success(function () {
							dialog.formDialog('close');
							table.squashTable().resetRowsByIds(ids);
						});
				}
			}

		})
});
