/*
 *     This file is part of the Redmine requirement connector for Squash TM (henix-tm-redmine-requirements) project.
 *     Copyright (C) 2017 - 2022 Henix, henix.fr - All Rights Reserved
 *
 *     Unauthorized copying of this file, via any medium is strictly prohibited
 *     Proprietary and confidential
 *
 * 	 (C)Henix. Tous droits réservés.
 *
 * 	Avertissement : ce programme est protégé par la loi relative au droit d'auteur et par les conventions internationales. Toute reproduction ou distribution partielle ou totale du logiciel, par quelque moyen que ce soit, est strictement interdite.
 */
define(["jquery", "domReady", "handlebars",  "squash.translator", "app/ws/squashtm.notification",  "tree", "serverauth/auth-manager",
        "text!redminereq/dialog-template.html!strip",
        "jquery.squash.formdialog", "app/squash.handlebars.helpers"],
        function($, domReady, Handlebars, translator, notification, zetree, authmanager, dialogTemplate){

	// *********** Prefiling the ajax with the required token (Squash 1.18+) ***********

	if ($("meta[name='_csrf']").length > 0){
        $.ajaxPrefilter(function (options, originalOptions, jqXHR) {
            var token = $("meta[name='_csrf']").attr("content");
            var header = $("meta[name='_csrf_header']").attr("content");
            jqXHR.setRequestHeader(header, token);
        });
    }

	var menuTemplate =
		'<li id="redminereq-synchronization" class="ui-state-disabled cursor-pointer">' +
			'<a> {{i18n "henix.redminereq.synchronization.menu.label" }} </a>' +
		'</li>';


	function init(){
		// add the menuitem in the import menu
		var menuHtml = Handlebars.compile(menuTemplate)();
		$("#tree-import-menu").append(menuHtml);
		var btn = $("#redminereq-synchronization");

		// add the dialog to the body
		var dialogHtml = Handlebars.compile(dialogTemplate)();
		$('body').append(dialogHtml);

		var dialog = $("#redminereq-synchronization-dialog");

		initDialog(dialog);


		// the button event handling
		// first ensure that the user is authenticated before opening the dialog
		btn.on('click', function(){
			var projectId = zetree.get().jstree('get_selected').attr('project');

			// get the serverId and challenge the user with it
			// if success -> open the dialog
			// that'll be some asynchronous api fest
			$.ajax({
				url : squashtm.app.contextRoot +'redminereq/project/'+projectId+'/configuration/server',
				type : 'GET',
				dataType : 'json'
			})
			.then(
      function(authBugTracker){
				var promise = authmanager.get(authBugTracker).authenticate();
				return promise;
			},
      function (error) {
			  notification.showError(translator.get("henix.redminereq.validation.invalidconfiguration"));
        //setting errorIsHandled at true in XHRrequest to avoid nasty blocking orange thing...
        //see line 65 in app/ws/squashtm.notification
        error.errorIsHandled = true;
			})
			.then(function(){
				dialog.formDialog('open');
			});

		});

		// rest of initialization
		domReady(function(){
			var tree = zetree.get();

			// refresh the state of the buttonmenu
			tree.on("select_node.jstree deselect_node.jstree deselect_all.jstree", function(){
				var nodes = tree.jstree('get_selected');
				if (isActive(nodes)){
					btn.removeClass("disabled ui-state-disabled");
				}
				else{
					btn.addClass("disabled ui-state-disabled");
				}
			});
		});
	}

	function initDialog(dialog){

		dialog.formDialog();

		dialog.on('formdialogopen', function(){
			dialog.formDialog("setState", "preview");
		});

		dialog.on('formdialogconfirm', function(){
			dialog.formDialog('setState', 'processing');
			var id = zetree.get().jstree('get_selected').attr('project');

			var xhr = $.ajax({
				url : squashtm.app.contextRoot + 'redminereq/synchronizator/newjob/project/'+id,
				type : 'POST',
				dataType : 'json'
			})
			.success(function(report){
				// ensure that the tree will be refreshed when dialog is closed
				dialog.one("formdialogclose", function(){
					zetree.get().jstree('get_selected').refresh();
				});

				if (report.allFetchFailed){
					dialog.formDialog('setState', 'summary-allfailed');
				} else if (report.failureOnFetch){
				    dialog.formDialog('setState', 'summary-so-so')
				} else {
					dialog.formDialog('setState', 'summary-allfine');
				}
			});
		});

		dialog.on('formdialogcancel', function(){
			dialog.formDialog('close');
		});

		dialog.on('formdialogdone', function(){
			dialog.formDialog('close');
		});

	}


	/*
	 * The button is active if all conditions are true :
	 * - only one node is selected,
	 * - that node is a project,
	 * - the user can manage that project
	 */
	function isActive(nodes){
		return (
			nodes.length === 1 &&
			nodes.filter(':library').length === nodes.length &&
			nodes.isAuthorized('IMPORT') &&
      nodes.isWorkspaceWizardEnabled({id : 'squash.tm.plugin.redminereq'})
		);
	}

	return {
		init : init
	};
});
